"""
This type stub file was generated by pyright.
"""

from typing import Generic, Optional, TypeVar, overload
from collections.abc import Iterable

from amaranth.hdl import MemoryInstance, Value, ValueCastable

from amaranth_types.types import ShapeCastable, ShapeLike, FlatShapeLike, ValueLike, FlatValueLike
from ..hdl import MemoryData, Signal
from . import wiring

__all__ = ["Memory", "ReadPort", "WritePort"]


_T_ValueOrValueCastable = TypeVar("_T_ValueOrValueCastable", bound=Value | ValueCastable, covariant=True)
_T_ShapeLike = TypeVar("_T_ShapeLike", bound=ShapeLike, covariant=True)
_T_FlatShapeLike = TypeVar("_T_FlatShapeLike", bound=FlatShapeLike, covariant=True)


class Memory(wiring.Component, Generic[_T_ShapeLike, _T_ValueOrValueCastable]):
    """Addressable array of rows.

    This :ref:`component <wiring>` is used to construct a memory array by first specifying its
    dimensions and initial contents using the :py:`shape`, :py:`depth`, and :py:`init` parameters,
    and then adding memory ports using the :meth:`read_port` and :meth:`write_port` methods.
    Because it is mutable, it should be created and used locally within
    the :ref:`elaborate <lang-elaboration>` method. It is an error to add ports to or change
    initial contents of a memory after it has been elaborated.

    The :py:`init` parameter and assignment to the :py:`init` attribute have the same effect, with
    :class:`Memory.Init` converting elements of the iterable to match :py:`shape` and using
    a default value for rows that are not explicitly initialized.

    .. warning::

        Uninitialized memories (including ASIC memories and some FPGA memories) are
        `not yet supported <https://github.com/amaranth-lang/amaranth/issues/270>`_, and
        the :py:`init` parameter must be always provided, if only as :py:`init=[]`.

    Parameters
    ----------
    shape : :ref:`shape-like <lang-shapelike>` object
        Shape of each memory row.
    depth : :class:`int`
        Number of memory rows.
    init : iterable of initial values
        Initial values for memory rows.

    Platform overrides
    ------------------
    Define the :py:`get_memory()` platform method to override the implementation of
    :class:`Memory`, e.g. to instantiate library cells directly.
    """
    Init = MemoryData.Init

    @overload
    def __init__(self: Memory[ShapeCastable[_T_ValueOrValueCastable], _T_ValueOrValueCastable], data: None = ..., *, shape: ShapeCastable[_T_ValueOrValueCastable] = ..., depth: int = ..., init: Iterable[ValueLike] = ..., attrs: Optional[dict[str, str]] = ..., src_loc_at: int = ...) -> None:
        ...

    @overload
    def __init__(self: Memory[_T_FlatShapeLike, Value], data: None = ..., *, shape: _T_FlatShapeLike = ..., depth: int = ..., init: Iterable[ValueLike] = ..., attrs: Optional[dict[str, str]] = ..., src_loc_at: int = ...) -> None:
        ...

    @overload
    def __init__(self: Memory[_T_ShapeLike, Value], data: MemoryData = ..., *, shape: None = ..., depth: None = ..., init: None = ..., attrs: Optional[dict[str, str]] = ..., src_loc_at: int = ...) -> None:
        ...

    def __init__(self, data: Optional[MemoryData] = None, *, shape: Optional[ShapeLike] = None, depth: Optional[int] = None, init: Optional[Iterable[ValueLike]] = None, attrs: Optional[dict[str, str]] = ..., src_loc_at: int = ...) -> None:
        ...
    
    @property
    def data(self) -> MemoryData:
        ...
    
    @property
    def shape(self) -> ShapeLike:
        ...
    
    @property
    def depth(self) -> int:
        ...
    
    @property
    def init(self) -> Init:
        ...
    
    @init.setter
    def init(self, init: Init) -> None:
        ...
    
    @property
    def attrs(self) -> dict[str, str]:
        ...
    
    def read_port(self, *, domain: str = ..., transparent_for: Iterable[WritePort] = ..., src_loc_at: int = ...) -> ReadPort[_T_ShapeLike, _T_ValueOrValueCastable]:
        """Request a read port.

        If :py:`domain` is :py:`"comb"`, the created read port is asynchronous and always enabled
        (with its enable input is tied to :py:`Const(1)`), and its data output always reflects
        the contents of the selected row. Otherwise, the created read port is synchronous,
        and its data output is updated with the contents of the selected row at each
        :ref:`active edge <lang-sync>` of :py:`domain` where the enable input is asserted.

        The :py:`transparent_for` parameter specifies the *transparency set* of this port: zero or
        more :class:`WritePort`\\ s, all of which must belong to the same memory and clock domain.
        If another port writes to a memory row at the same time as this port reads from the same
        memory row, and that write port is a part of the transparency set, then this port retrieves
        the new contents of the row; otherwise, this port retrieves the old contents of the row.

        If another write port belonging to a different clock domain updates a memory row that this
        port is reading at the same time, the behavior is undefined.

        The signature of the returned port is
        :py:`ReadPort.Signature(shape=self.shape, addr_width=ceil_log2(self.depth))`.

        Returns
        -------
        :class:`ReadPort`
        """
        ...
    
    def write_port(self, *, domain: str = ..., granularity: Optional[int] = ..., src_loc_at: int = ...) -> WritePort[_T_ShapeLike, _T_ValueOrValueCastable]:
        """Request a write port.

        The created write port is synchronous, updating the contents of the selected row at each
        :ref:`active edge <lang-sync>` of :py:`domain` where the enable input is asserted.

        Specifying a *granularity* when :py:`shape` is :func:`unsigned(width) <.unsigned>` or
        :class:`data.ArrayLayout(_, width) <.data.ArrayLayout>` makes it possible to partially
        update a memory row. In this case, :py:`granularity` must be an integer that evenly divides
        :py:`width`, and the memory row is split into :py:`width // granularity` equally sized
        parts, each of which is updated if the corresponding bit of the enable input is asserted.

        The signature of the new port is
        :py:`WritePort.Signature(shape=self.shape, addr_width=ceil_log2(self.depth), granularity=granularity)`.

        Returns
        -------
        :class:`WritePort`
        """
        ...
    
    @property
    def read_ports(self) -> tuple[ReadPort, ...]:
        """All read ports defined so far.

        This property is provided for the :py:`platform.get_memory()` override.
        """
        ...
    
    @property
    def write_ports(self) -> tuple[WritePort, ...]:
        """All write ports defined so far.

        This property is provided for the :py:`platform.get_memory()` override.
        """
        ...
    
    def elaborate(self, platform) -> MemoryInstance:
        ...
    


class ReadPort(Generic[_T_ShapeLike, _T_ValueOrValueCastable]):
    """A read memory port.

    Memory read ports, which are :ref:`interface objects <wiring>`, can be constructed by calling
    :meth:`Memory.read_port` or via :meth:`ReadPort.Signature.create() <.Signature.create>`.

    An asynchronous (:py:`"comb"` domain) memory read port is always enabled. The :py:`en` input of
    such a port is tied to :py:`Const(1)`.

    Attributes
    ----------
    signature : :class:`ReadPort.Signature`
        Signature of this memory port.
    memory : :class:`Memory` or :py:`None`
        Memory associated with this memory port.
    domain : :class:`str`
        Name of this memory port's clock domain. For asynchronous ports, :py:`"comb"`.
    transparent_for : :class:`tuple` of :class:`WritePort`
        Transparency set of this memory port.
    """
    class Signature(wiring.Signature):
        """Signature of a memory read port.

        Parameters
        ----------
        addr_width : :class:`int`
            Width of the address port.
        shape : :ref:`shape-like <lang-shapelike>` object
            Shape of the data port.

        Members
        -------
        en: :py:`In(1, init=1)`
            Enable input.
        addr: :py:`In(addr_width)`
            Address input.
        data: :py:`Out(shape)`
            Data output.
        """
        def __init__(self, *, addr_width: int, shape: _T_ShapeLike) -> None:
            ...
        
        def create(self, *, path: tuple[str | int, ...] = ..., src_loc_at: int =...) -> ReadPort[_T_ShapeLike, _T_ValueOrValueCastable]:
            """:meta private:"""
            ...
        
        @property
        def addr_width(self) -> int:
            ...
        
        @property
        def shape(self) -> _T_ShapeLike:
            ...
        
        def __eq__(self, other) -> bool:
            ...
        
        def __repr__(self) -> str:
            ...

    @property
    def en(self) -> Signal:
        ...

    @property
    def addr(self) -> Signal:
        ...

    @property
    def data(self) -> _T_ValueOrValueCastable:
        ...

    def __init__(self, signature: Signature, *, memory: Memory, domain: str, transparent_for: Iterable[WritePort] = ..., path: tuple[str | int, ...] = ..., src_loc_at: int = ...) -> None:
        ...
    
    @property
    def signature(self) -> Signature:
        ...
    
    @property
    def memory(self) -> Memory[_T_ShapeLike, _T_ValueOrValueCastable]:
        ...
    
    @property
    def domain(self) -> str:
        ...
    
    @property
    def transparent_for(self) -> tuple[WritePort, ...]:
        ...
    


class WritePort(Generic[_T_ShapeLike, _T_ValueOrValueCastable]):
    """A write memory port.

    Memory write ports, which are :ref:`interface objects <wiring>`, can be constructed by calling
    :meth:`Memory.write_port` or via :meth:`WritePort.Signature.create() <.Signature.create>`.

    Attributes
    ----------
    signature : :class:`WritePort.Signature`
        Signature of this memory port.
    memory : :class:`Memory` or :py:`None`
        Memory associated with this memory port.
    domain : :class:`str`
        Name of this memory port's clock domain. Never :py:`"comb"`.
    """
    class Signature(wiring.Signature):
        """Signature of a memory write port.

        Width of the enable input is determined as follows:

        * If :py:`granularity` is :py:`None`,
          then :py:`en_width == 1`.
        * If :py:`shape` is :func:`unsigned(data_width) <.unsigned>`,
          then :py:`en_width == data_width // granularity`.
        * If :py:`shape` is :class:`data.ArrayLayout(_, elem_count) <.data.ArrayLayout>`,
          then :py:`en_width == elem_count // granularity`.

        Parameters
        ----------
        addr_width : :class:`int`
            Width of the address port.
        shape : :ref:`shape-like <lang-shapelike>` object
            Shape of the data port.
        granularity : :class:`int` or :py:`None`
            Granularity of memory access.

        Members
        -------
        en: :py:`In(en_width)`
            Enable input.
        addr: :py:`In(addr_width)`
            Address input.
        data: :py:`In(shape)`
            Data input.
        """
        def __init__(self, *, addr_width: int, shape: _T_ShapeLike, granularity: Optional[int] = ...) -> None:
            ...
        
        def create(self, *, path: tuple[str | int, ...] = ..., src_loc_at: int = ...) -> WritePort[_T_ShapeLike, _T_ValueOrValueCastable]:
            """:meta private:"""
            ...
        
        @property
        def addr_width(self) -> int:
            ...
        
        @property
        def shape(self) -> _T_ShapeLike:
            ...
        
        @property
        def granularity(self) -> Optional[int]:
            ...
        
        def __eq__(self, other) -> bool:
            ...
        
        def __repr__(self) -> str:
            ...

    @property
    def en(self) -> Signal:
        ...

    @property
    def addr(self) -> Signal:
        ...

    @property
    def data(self) -> _T_ValueOrValueCastable:
        ...
    
    def __init__(self, signature, *, memory, domain, path=..., src_loc_at=...) -> None:
        ...
    
    @property
    def signature(self) -> Signature:
        ...
    
    @property
    def memory(self) -> Memory[_T_ShapeLike, _T_ValueOrValueCastable]:
        ...
    
    @property
    def domain(self) -> str:
        ...





"""
This type stub file was generated by pyright.
"""

import enum
from typing import Iterable

"""
This type stub file was generated by pyright.
"""
__all__ = ["Net", "Value", "IONet", "IOValue", "FormatValue", "Format", "Netlist", "ModuleNetFlow", "IODirection", "Module", "Cell", "Top", "Operator", "Part", "Matches", "PriorityMatch", "Assignment", "AssignmentList", "FlipFlop", "Memory", "SyncWritePort", "AsyncReadPort", "SyncReadPort", "AsyncPrint", "SyncPrint", "Initial", "AnyValue", "AsyncProperty", "SyncProperty", "Instance", "IOBuffer"]
class Net(int):
    __slots__ = ...
    @classmethod
    def from_cell(cls, cell: int, bit: int):
        ...
    
    @classmethod
    def from_const(cls, val: int):
        ...
    
    @classmethod
    def from_late(cls, val: int):
        ...
    
    @property
    def is_const(self):
        ...
    
    @property
    def const(self):
        ...
    
    @property
    def is_late(self):
        ...
    
    @property
    def is_cell(self):
        ...
    
    @property
    def cell(self):
        ...
    
    @property
    def bit(self):
        ...
    
    @classmethod
    def ensure(cls, value: Net):
        ...
    
    def __repr__(self):
        ...
    
    __str__ = ...


class Value(tuple):
    __slots__ = ...
    def __new__(cls, nets: Net | Iterable[Net] = ...):
        ...
    
    @classmethod
    def from_const(cls, value, width):
        ...
    
    @classmethod
    def zeros(cls, digits=...):
        ...
    
    @classmethod
    def ones(cls, digits=...):
        ...
    
    def __getitem__(self, index):
        ...
    
    def __repr__(self):
        ...
    
    @property
    def is_const(self):
        ...
    
    __str__ = ...


class IONet(int):
    __slots__ = ...
    @classmethod
    def from_port(cls, port: int, bit: int):
        ...
    
    @property
    def port(self):
        ...
    
    @property
    def bit(self):
        ...
    
    @classmethod
    def ensure(cls, value: IONet):
        ...
    
    def __repr__(self):
        ...
    
    __str__ = ...


class IOValue(tuple):
    __slots__ = ...
    def __new__(cls, nets: IONet | Iterable[IONet] = ...):
        ...
    
    def __getitem__(self, index):
        ...
    
    def __repr__(self):
        ...
    
    __str__ = ...


class FormatValue:
    """A single formatted value within ``Format``.

    Attributes
    ----------

    value: Value
    format_desc: str
    signed: bool
    """
    def __init__(self, value, format_desc, *, signed) -> None:
        ...
    
    def __repr__(self):
        ...
    


class Format:
    """Like _ast.Format, but for NIR.

    Attributes
    ----------

    chunks: tuple of str and FormatValue
    """
    def __init__(self, chunks) -> None:
        ...
    
    def __repr__(self):
        ...
    
    def input_nets(self):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    


class Netlist:
    """A fine netlist. Consists of:

    - a flat array of cells
    - a dictionary of connections for late-bound nets
    - a map of hierarchical names to nets
    - a map of signals to nets

    The nets are virtual: a list of nets is not materialized anywhere in the netlist.
    A net is a single bit wide and represented as a single int. The int is encoded as follows:

    - A negative number means a late-bound net. The net should be looked up in the ``connections``
      dictionary to find its driver.
    - Non-negative numbers are cell outputs, and are split into bitfields as follows:

      - bits 0-15: output bit index within a cell (exact meaning is cell-dependent)
      - bits 16-...: index of cell in ``netlist.cells``

    Cell 0 is always ``Top``.  The first two output bits of ``Top`` are considered to be constants
    ``0`` and ``1``, which effectively means that net encoded as ``0`` is always a constant ``0`` and
    net encoded as ``1`` is always a constant ``1``.

    Multi-bit values are represented as tuples of int.

    Attributes
    ----------

    modules : list of ``Module``
    cells : list of ``Cell``
    connections : dict of (negative) int to int
    io_ports : list of ``IOPort``
    signals : dict of Signal to ``Value``
    last_late_net: int
    """
    def __init__(self) -> None:
        ...
    
    def resolve_net(self, net: Net):
        ...
    
    def resolve_value(self, value: Value):
        ...
    
    def resolve_all_nets(self):
        ...
    
    def __repr__(self):
        ...
    
    def add_module(self, parent, name: str, *, src_loc=..., cell_src_loc=...):
        ...
    
    def add_cell(self, cell):
        ...
    
    def add_value_cell(self, width: int, cell):
        ...
    
    def alloc_late_value(self, width: int):
        ...
    
    @property
    def top(self):
        ...
    


class ModuleNetFlow(enum.Enum):
    """Describes how a given Net flows into or out of a Module.

    The net can also be none of these (not present in the dictionary at all),
    when it is not present in the module at all.
    """
    Internal = ...
    Input = ...
    Output = ...


class IODirection(enum.Enum):
    Input = ...
    Output = ...
    Bidir = ...
    def __or__(self, other):
        ...
    


class Module:
    """A module within the netlist.

    Attributes
    ----------

    parent: index of parent module, or ``None`` for top module
    name: a tuple of str, hierarchical name of this module (top has empty tuple)
    src_loc: str
    submodules: a list of nested module indices
    signal_names: a SignalDict from Signal to str, signal names visible in this module
    net_flow: a dict from Net to NetFlow, describes how a net is used within this module
    ports: a dict from port name to (Value, ModuleNetFlow) pair
    io_ports: a dict from port name to (IOValue, IODirection) pair
    cells: a list of cell indices that belong to this module
    """
    def __init__(self, parent, name, *, src_loc, cell_src_loc) -> None:
        ...
    


class Cell:
    """A base class for all cell types.

    Attributes
    ----------

    src_loc: str
    module: int, index of the module this cell belongs to (within Netlist.modules)
    """
    def __init__(self, module_idx: int, *, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def io_nets(self):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    


class Top(Cell):
    """A special cell type representing top-level non-IO ports. Must be present in the netlist exactly
    once, at index 0.

    Top-level outputs are stored as a dict of names to their assigned values.

    Top-level inputs are effectively the output of this cell. They are stored
    as a dict of names to a (start bit index, width) tuple. Output bit indices 0 and 1 are reserved
    for constant nets, so the lowest bit index that can be assigned to a port is 2.

    Attributes
    ----------

    ports_o: dict of str to Value
    ports_i: dict of str to (int, int)
    """
    def __init__(self) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Operator(Cell):
    """Roughly corresponds to ``hdl.ast.Operator``.

    The available operators are roughly the same as in AST, with some changes:

    - '<', '>', '<=', '>=', '//', '%', '>>' have signed and unsigned variants that are selected
      by prepending 'u' or 's' to operator name
    - 's', 'u', and unary '+' are redundant and do not exist
    - many operators restrict input widths to be the same as output width,
      and/or to be the same as each other

    The unary operators are:

    - '-', '~': like AST, input same width as output
    - 'b', 'r|', 'r&', 'r^': like AST, 1-bit output

    The binary operators are:

    - '+', '-', '*', '&', '^', '|', 'u//', 's//', 'u%', 's%': like AST, both inputs same width as output
    - '<<', 'u>>', 's>>': like AST, first input same width as output
    - '==', '!=', 'u<', 's<', 'u>', 's>', 'u<=', 's<=', 'u>=', 's>=': like AST, both inputs need to have
      the same width, 1-bit output

    The ternary operators are:

    - 'm': multiplexer, first input needs to have width of 1, second and third operand need to have 
      the same width as output; implements arg0 ? arg1 : arg2

    Attributes
    ----------

    operator: str, symbol of the operator (from the above list)
    inputs: tuple of Value
    """
    def __init__(self, module_idx, *, operator: str, inputs, src_loc) -> None:
        ...
    
    @property
    def width(self):
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Part(Cell):
    """Corresponds to ``hdl.ast.Part``.

    Attributes
    ----------

    value: Value, the data input
    value_signed: bool
    offset: Value, the offset input
    width: int
    stride: int
    """
    def __init__(self, module_idx, *, value, value_signed, offset, width, stride, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Matches(Cell):
    """A combinatorial cell performing a comparison like ``Value.matches``
    (or, equivalently, a case condition).

    Attributes
    ----------

    value: Value
    patterns: tuple of str, each str contains '0', '1', '-'
    """
    def __init__(self, module_idx, *, value, patterns, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class PriorityMatch(Cell):
    """Used to represent a single switch on the control plane of processes.

    The output is the same length as ``inputs``. If ``en`` is ``0``, the output
    is all-0. Otherwise, output keeps the lowest-numbered ``1`` bit in the input
    (if any) and masks all other bits to ``0``.

    Note: the RTLIL backend requires all bits of ``inputs`` to be driven
    by a ``Match`` cell within the same module.

    Attributes
    ----------
    en: Net
    inputs: Value
    """
    def __init__(self, module_idx, *, en, inputs, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Assignment:
    """A single assignment in an ``AssignmentList``.

    The assignment is executed iff ``cond`` is true. When the assignment
    is executed, ``len(value)`` bits starting at position `offset` are set
    to the value ``value``, and the remaining bits are unchanged.
    Assignments to out-of-bounds bit indices are ignored.

    Attributes
    ----------

    cond: Net
    start: int
    value: Value
    src_loc: str
    """
    def __init__(self, *, cond, start, value, src_loc) -> None:
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class AssignmentList(Cell):
    """Used to represent a single assigned signal on the data plane of processes.

    The output of this cell is determined by starting with the ``default`` value,
    then executing each assignment in sequence.

    Note: the RTLIL backend requires all ``cond`` inputs of assignments to be driven
    by a ``PriorityMatch`` cell within the same module.

    Attributes
    ----------
    default: Value
    assignments: tuple of ``Assignment``
    """
    def __init__(self, module_idx, *, default, assignments, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class FlipFlop(Cell):
    """A flip-flop. ``data`` is the data input. ``init`` is the initial and async reset value.
    ``clk`` and ``clk_edge`` work as in a ``ClockDomain``. ``arst`` is the async reset signal,
    or ``0`` if async reset is not used.

    Attributes
    ----------

    data: Value
    init: int
    clk: Net
    clk_edge: str, either 'pos' or 'neg'
    arst: Net
    attributes: dict from str to int, Const, or str
    """
    def __init__(self, module_idx, *, data, init, clk, clk_edge, arst, attributes, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Memory(Cell):
    """Corresponds to ``Memory``.  ``init`` must have length equal to ``depth``.
    Read and write ports are separate cells.

    Attributes
    ----------

    width: int
    depth: int
    init: tuple of int
    name: str
    attributes: dict from str to int, Const, or str
    """
    def __init__(self, module_idx, *, width, depth, init, name, attributes, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class SyncWritePort(Cell):
    """A single write port of a memory.  This cell has no output.

    Attributes
    ----------

    memory: cell index of ``Memory``
    data: Value
    addr: Value
    en: Value
    clk: Net
    clk_edge: str, either 'pos' or 'neg'
    """
    def __init__(self, module_idx, memory, *, data, addr, en, clk, clk_edge, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class AsyncReadPort(Cell):
    """A single asynchronous read port of a memory.

    Attributes
    ----------

    memory: cell index of ``Memory``
    width: int
    addr: Value
    """
    def __init__(self, module_idx, memory, *, width, addr, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class SyncReadPort(Cell):
    """A single synchronous read port of a memory.  The cell output is the data port.
    ``transparent_for`` is the set of write ports (identified by cell index) that this
    read port is transparent with.

    Attributes
    ----------

    memory: cell index of ``Memory``
    width: int
    addr: Value
    en: Net
    clk: Net
    clk_edge: str, either 'pos' or 'neg'
    transparent_for: tuple of int
    """
    def __init__(self, module_idx, memory, *, width, addr, en, clk, clk_edge, transparent_for, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class AsyncPrint(Cell):
    """Corresponds to ``Print`` in the "comb" domain.

    Attributes
    ----------

    en: Net
    format: Format
    """
    def __init__(self, module_idx, *, en, format, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class SyncPrint(Cell):
    """Corresponds to ``Print`` in domains other than "comb".

    Attributes
    ----------

    en: Net
    clk: Net
    clk_edge: str, either 'pos' or 'neg'
    format: Format
    """
    def __init__(self, module_idx, *, en, clk, clk_edge, format, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Initial(Cell):
    """Corresponds to ``Initial`` value."""
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class AnyValue(Cell):
    """Corresponds to ``AnyConst`` or ``AnySeq``. ``kind`` must be either ``'anyconst'``
    or ``'anyseq'``.

    Attributes
    ----------

    kind: str, 'anyconst' or 'anyseq'
    width: int
    """
    def __init__(self, module_idx, *, kind, width, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class AsyncProperty(Cell):
    """Corresponds to ``Assert``, ``Assume``, or ``Cover`` in the "comb" domain.

    Attributes
    ----------

    kind: str, either 'assert', 'assume', or 'cover'
    test: Net
    en: Net
    format: Format or None
    """
    def __init__(self, module_idx, *, kind, test, en, format, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class SyncProperty(Cell):
    """Corresponds to ``Assert``, ``Assume``, or ``Cover`` in domains other than "comb".

    Attributes
    ----------

    kind: str, either 'assert', 'assume', or 'cover'
    test: Net
    en: Net
    clk: Net
    clk_edge: str, either 'pos' or 'neg'
    format: Format or None
    """
    def __init__(self, module_idx, *, kind, test, en, clk, clk_edge, format, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class Instance(Cell):
    """Corresponds to ``Instance``. ``type``, ``parameters`` and ``attributes`` work the same as in
    ``Instance``. Input and inout ports are represented as a dict of port names to values.
    Inout ports must be connected to nets corresponding to an IO port of the ``Top`` cell.

    Output ports are represented as a dict of port names to (start bit index, width) describing
    their position in the virtual "output" of this cell.

    Attributes
    ----------

    type: str
    name: str
    parameters: dict of str to Const, int, or str
    attributes: dict of str to Const, int, or str
    ports_i: dict of str to Value
    ports_o: dict of str to pair of int (index start, width)
    ports_io: dict of str to (IOValue, IODirection)
    """
    def __init__(self, module_idx, *, type, name, parameters, attributes, ports_i, ports_o, ports_io, src_loc) -> None:
        ...
    
    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def io_nets(self):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    


class IOBuffer(Cell):
    """An IO buffer cell. This cell does two things:

    - a tristate buffer is inserted driving ``port`` based on ``o`` and ``oe`` nets (output buffer)
    - the value of ``port`` is sampled and made available as output of this cell (input buffer)

    Attributes
    ----------

    port: IOValue
    dir: IODirection
    o: Value or None
    oe: Net or None
    """

    @property
    def port(self) -> IOValue:
        ...

    @property
    def dir(self) -> IODirection:
        ...

    @property
    def o(self) -> Value | None:
        ...

    @property
    def oe(self) -> Net | None:
        ...

    def __init__(self, module_idx, *, port, dir, o=..., oe=..., src_loc) -> None:
        ...

    def input_nets(self):
        ...
    
    def output_nets(self, self_idx: int):
        ...
    
    def io_nets(self):
        ...
    
    def resolve_nets(self, netlist: Netlist):
        ...
    
    def __repr__(self):
        ...
    






from collections.abc import Iterable
from amaranth import *
from typing import Any, Generic, Optional, Protocol, TypeVar
from amaranth.hdl import ValueCastable

from amaranth_types.types import HasElaborate, ShapeLike, ValueLike


__all__ = ["AbstractMemory", "AbstractReadPort", "AbstractWritePort", "AbstractMemoryConstructor"]

_T_ValueOrValueCastable = TypeVar("_T_ValueOrValueCastable", bound=Value | ValueCastable, covariant=True)
_T_Cov_ShapeLike = TypeVar("_T_Cov_ShapeLike", bound=ShapeLike, covariant=True)
_T_ShapeLike = TypeVar("_T_ShapeLike", bound=ShapeLike)


class AbstractReadPort(Generic[_T_Cov_ShapeLike, _T_ValueOrValueCastable], Protocol):
    @property
    def en(self) -> Signal:
        ...

    @property
    def addr(self) -> Signal:
        ...

    @property
    def data(self) -> _T_ValueOrValueCastable:
        ...


class AbstractWritePort(Generic[_T_Cov_ShapeLike, _T_ValueOrValueCastable], Protocol):
    @property
    def en(self) -> Signal:
        ...

    @property
    def addr(self) -> Signal:
        ...

    @property
    def data(self) -> _T_ValueOrValueCastable:
        ...


class AbstractMemory(Generic[_T_Cov_ShapeLike, _T_ValueOrValueCastable], HasElaborate, Protocol):
    @property
    def shape(self) -> ShapeLike:
        ...

    @property
    def depth(self) -> int:
        ...

    def read_port(
        self,
        *,
        domain: str = ...,
        transparent_for: Iterable[Any] = ...,
        src_loc_at: int = ...
    ) -> AbstractReadPort[_T_Cov_ShapeLike, _T_ValueOrValueCastable]:
        ...

    def write_port(
        self,
        *,
        domain: str = ...,
        granularity: Optional[int] = ...,
        src_loc_at: int = ...
    ) -> AbstractWritePort[_T_Cov_ShapeLike, _T_ValueOrValueCastable]:
        ...


class AbstractMemoryConstructor(Generic[_T_ShapeLike, _T_ValueOrValueCastable], Protocol):
    @staticmethod
    def __call__(
        *,
        shape: _T_ShapeLike,
        depth: int,
        init: Iterable[ValueLike],
        attrs: Optional[dict[str, str]] = ...,
        src_loc_at: int = ...
    ) -> AbstractMemory[_T_ShapeLike, _T_ValueOrValueCastable]:
        ...



"""
This type stub file was generated by pyright.
"""

from abc import ABCMeta, abstractmethod
from collections.abc import Iterator, Mapping, Sequence
from typing import Any, Optional, TypeVar, Generic, Self
from amaranth.hdl import *
import amaranth.hdl as hdl
from amaranth.hdl._ast import Assign, ShapeCastable, ValueCastable, Format, _FormatLike
from amaranth_types import ShapeLike, ValueLike

__all__ = ["Field", "Layout", "StructLayout", "UnionLayout", "ArrayLayout", "FlexibleLayout", "View", "Const", "Struct", "Union"]


_T_ShapeCastable = TypeVar("_T_ShapeCastable", bound=ShapeCastable, covariant=True)


class Field:
    def __init__(self, shape: ShapeLike, offset: int) -> None:
        ...
    
    @property
    def shape(self) -> ShapeLike:
        ...
    
    @property
    def offset(self) -> int:
        ...
    
    @property
    def width(self) -> int:
        ...
    
    def __eq__(self, other) -> bool:
        ...
    
    def __repr__(self) -> str:
        ...
    


class Layout(ShapeCastable[View[Self]], metaclass=ABCMeta):
    @staticmethod
    def cast(obj: ShapeCastable) -> Layout:
        """Cast a shape-castable object to a layout."""
        ...
    
    @abstractmethod
    def __iter__(self) -> Iterator[tuple[int | str, Field]]:
        """Iterate the layout, yielding ``(key, field)`` pairs. Keys may be strings or integers."""
        ...
    
    @abstractmethod
    def __getitem__(self, key: int | str) -> Field:
        """Retrieve the :class:`Field` associated with the ``key``, or raise ``KeyError``."""
        ...
    
    def size(self) -> int:
        """Size of the layout."""
        ...

    def as_shape(self) -> Shape:
        """Convert the representation defined by the layout to an unsigned :class:`Shape`."""
        ...
    
    def __eq__(self, other) -> bool:
        """Compare the layout with another.

        Two layouts are equal if they have the same size and the same fields under the same names.
        The order of the fields is not considered.
        """
        ...

    def __call__(self, target: ValueLike) -> View[Self]:
        """Create a view into a target."""
        ...

    def const(self, init: Optional[Sequence[Any] | Mapping[int | str, Any]]) -> Const:
        """Convert a constant initializer to a constant."""
        ...

    def from_bits(self, raw: int) -> Const:
        ...

    def format(self, value: ValueLike, format_spec: str) -> Format.Struct:
        ...


class StructLayout(Layout):
    def __init__(self, members: Mapping[str, ShapeLike]) -> None:
        ...
    
    @property
    def members(self) -> dict[str, ShapeLike]:
        ...
    
    def __iter__(self) -> Iterator[tuple[str, Field]]:
        ...
    
    def __getitem__(self, key: str) -> Field:
        ...
    
    @property
    def size(self) -> int:
        ...
    
    def __repr__(self) -> str:
        ...
    


class UnionLayout(Layout):
    def __init__(self, members: Mapping[str, ShapeLike]) -> None:
        ...
    
    @property
    def members(self) -> dict[str, ShapeLike]:
        ...
    
    def __iter__(self) -> Iterator[tuple[str, Field]]:
        ...
    
    def __getitem__(self, key: str) -> Field:
        ...
    
    @property
    def size(self) -> int:
        ...
    
    def __repr__(self) -> str:
        ...
    


class ArrayLayout(Layout):
    def __init__(self, elem_shape: ShapeLike, length: int) -> None:
        ...
    
    @property
    def elem_shape(self) -> ShapeLike:
        ...
    
    @property
    def length(self) -> int:
        ...
    
    def __iter__(self) -> Iterator[tuple[int, Field]]:
        ...
    
    def __getitem__(self, key: int) -> Field:
        ...
    
    @property
    def size(self) -> int:
        ...
    
    def __repr__(self) -> str:
        ...

    def format(self, value: ValueLike, format_spec: str) -> Format.Array:
        ...
    


class FlexibleLayout(Layout):
    def __init__(self, size: int, fields: Mapping[int | str, Field]) -> None:
        ...
    
    @property
    def size(self) -> int:
        ...
    
    @property
    def fields(self) -> dict[int | str, Field]:
        ...
    
    def __iter__(self) -> Iterator[tuple[int | str, Field]]:
        ...
    
    def __getitem__(self, key: int | str) -> Field:
        ...
    
    def __repr__(self) -> str:
        ...
    


class View(ValueCastable, Generic[_T_ShapeCastable]):
    def __init__(self, layout: _T_ShapeCastable, target: ValueLike) -> None:
        ...
    
    @ValueCastable.lowermethod
    def as_value(self) -> Value:
        ...
    
    def shape(self) -> _T_ShapeCastable:
        ...

    def eq(self, other: ValueLike) -> Assign:
        ...
    
    def __getitem__(self, key: str | ValueLike):  # having a defined type hard to work with
        ...
    
    def __getattr__(self, name: str):  # having a defined type hard to work with
        ...
   
    def __len__(self) -> int:
        ...

    def __eq__(self, other) -> bool:
        ...


class _AggregateMeta(ShapeCastable[Self], type):
    def __new__(metacls, name, bases, namespace, *, _layout_cls=..., **kwargs): # -> Self@_AggregateMeta:
        ...
    
    def as_shape(cls):
        ...

    def const(cls, init):
        ...

    def from_bits(cls, bits):
        ...

    def format(self, value: ValueLike, format_spec: str) -> _FormatLike:
        ...


class Const(ValueCastable, Generic[_T_ShapeCastable]):
    def __init__(self, layout: _T_ShapeCastable, target: int) -> None:
        ...

    def shape(self) -> _T_ShapeCastable:
        ...

    def as_bits(self) -> int:
        ...

    def as_value(self) -> hdl.Const:
        ...

    def __getitem__(self, key: int | str | ValueCastable) -> Any:
        ...

    def __getattr__(self, name: str) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __eq__(self, other) -> bool:
        ...

    def __ne__(self, other) -> bool:
        ...

    def __repr__(self) -> str:
        ...


class Struct(View, metaclass=_AggregateMeta):
    ...


class Union(View, metaclass=_AggregateMeta):
    ...





"""
This type stub file was generated by pyright.
"""

from .._toolchain.yosys import *
from ..hdl._ast import SignalDict

__all__ = ["YosysError", "convert", "convert_fragment"]
def convert_fragment(*args, strip_internal_attrs=..., **kwargs) -> tuple[str, SignalDict]:
    ...

def convert(elaboratable, name=..., platform=..., *, ports=..., emit_src=..., strip_internal_attrs=..., **kwargs) -> str:
    ...




"""
This type stub file was generated by pyright.
"""

from .._utils import deprecated
from ..hdl._cd import *
from ..hdl._ir import *

__all__ = ["Command", "Settle", "Delay", "Tick", "Passive", "Active"]
class Command:
    ...


class Settle(Command):
    def __repr__(self) -> str:
        ...
    


class Delay(Command):
    def __init__(self, interval=...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    


class Tick(Command):
    domain: str | ClockDomain

    def __init__(self, domain: str | ClockDomain = ...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    

class Passive(Command):
    def __repr__(self) -> str:
        ...
    


class Active(Command):
    def __repr__(self) -> str:
        ...
    





"""
This type stub file was generated by pyright.
"""

__all__ = ["ceil_log2", "exact_log2", "bits_for"]

def bits_for(n:int, require_sign_bit:bool=...) -> int:
    ...


def ceil_log2(n : int) -> int:
    """Returns the integer log2 of the smallest power-of-2 greater than or equal to ``n``.

    Raises a ``ValueError`` for negative inputs.
    """
    ...

def exact_log2(n : int) -> int:
    """Returns the integer log2 of ``n``, which must be an exact power of two.

    Raises a ``ValueError`` if ``n`` is not a power of two.
    """
    ...



"""
This type stub file was generated by pyright.
"""

from abc import ABCMeta, abstractmethod
from typing import Generic, TypeVar

__all__ = ["InvalidSchema", "InvalidAnnotation", "Annotation"]


_T = TypeVar("_T")


class InvalidSchema(Exception):
    """Exception raised when a subclass of :class:`Annotation` is defined with a non-conformant
    :data:`~Annotation.schema`."""
    ...


class InvalidAnnotation(Exception):
    """Exception raised by :meth:`Annotation.validate` when the JSON representation of
    an annotation does not conform to its schema."""
    ...


class Annotation(Generic[_T], metaclass=ABCMeta):
    """Interface annotation.

    Annotations are containers for metadata that can be retrieved from an interface object using
    the :meth:`Signature.annotations <.wiring.Signature.annotations>` method.

    Annotations have a JSON representation whose structure is defined by the `JSON Schema`_
    language.
    """
    schema: dict = ...
    def __init_subclass__(cls, **kwargs) -> None:
        """
        Defining a subclass of :class:`Annotation` causes its :data:`schema` to be validated.

        Raises
        ------
        :exc:`InvalidSchema`
            If :data:`schema` doesn't conform to the `2020-12` draft of `JSON Schema`_.
        :exc:`InvalidSchema`
            If :data:`schema` doesn't have a  `"$id" keyword`_ at its root. This requirement is
            specific to :class:`Annotation` schemas.
        """
        ...
    
    @property
    @abstractmethod
    def origin(self) -> _T:
        """Python object described by this :class:`Annotation` instance.

        Subclasses of :class:`Annotation` must implement this property.
        """
        ...
    
    @abstractmethod
    def as_json(self) -> dict:
        """Convert to a JSON representation.

        Subclasses of :class:`Annotation` must implement this method.

        JSON representation returned by this method must adhere to :data:`schema` and pass
        validation by :meth:`validate`.

        Returns
        -------
        :class:`dict`
            JSON representation of this annotation, expressed in Python primitive types
            (:class:`dict`, :class:`list`, :class:`str`, :class:`int`, :class:`bool`).
        """
        ...
    
    @classmethod
    def validate(cls, instance: dict) -> None:
        """Validate a JSON representation against :attr:`schema`.

        Arguments
        ---------
        instance : :class:`dict`
            JSON representation to validate, either previously returned by :meth:`as_json`
            or retrieved from an external source.

        Raises
        ------
        :exc:`InvalidAnnotation`
            If :py:`instance` doesn't conform to :attr:`schema`.
        """
        ...
    
    def __repr__(self) -> str:
        ...
    





"""
This type stub file was generated by pyright.
"""

from abc import ABCMeta, abstractmethod
from .._toolchain import *
from ..hdl import *
from .res import *
from .run import *

__all__ = ["Platform", "TemplatedPlatform"]
class Platform(ResourceManager, metaclass=ABCMeta):
    resources = ...
    connectors = ...
    default_clk = ...
    default_rst = ...
    required_tools = ...
    def __init__(self) -> None:
        ...
    
    @property
    def default_clk_constraint(self):
        ...
    
    @property
    def default_clk_frequency(self):
        ...
    
    def add_file(self, filename, content): # -> None:
        ...
    
    def iter_files(self, *suffixes): # -> Generator[Unknown, Any, None]:
        ...
    
    def build(self, elaboratable, name=..., build_dir=..., do_build=..., program_opts=..., do_program=..., **kwargs): # -> None:
        ...
    
    def has_required_tools(self): # -> bool:
        ...
    
    def create_missing_domain(self, name): # -> Module | None:
        ...
    
    def prepare(self, elaboratable, name=..., **kwargs):
        ...

    def iter_port_constraints_bits(self):
        ...

    @abstractmethod
    def toolchain_prepare(self, fragment, name, **kwargs):
        """
        Convert the ``fragment`` and constraints recorded in this :class:`Platform` into
        a :class:`BuildPlan`.
        """
        ...
    
    def toolchain_program(self, products, name, **kwargs):
        """
        Extract bitstream for fragment ``name`` from ``products`` and download it to a target.
        """
        ...
    
    def get_input(self, pin, port, attrs, invert): # -> Module:
        ...
    
    def get_output(self, pin, port, attrs, invert): # -> Module:
        ...
    
    def get_tristate(self, pin, port, attrs, invert): # -> Module:
        ...
    
    def get_input_output(self, pin, port, attrs, invert): # -> Module:
        ...
    
    def get_diff_input(self, pin, port, attrs, invert): # -> None:
        ...
    
    def get_diff_output(self, pin, port, attrs, invert): # -> None:
        ...
    
    def get_diff_tristate(self, pin, port, attrs, invert): # -> None:
        ...
    
    def get_diff_input_output(self, pin, port, attrs, invert): # -> None:
        ...
    


class TemplatedPlatform(Platform):
    toolchain = ...
    file_templates = ...
    command_templates = ...
    build_script_templates = ...
    def iter_clock_constraints(self): # -> Generator[tuple[Unknown, Unknown | None, Unknown], Any, None]:
        ...
    
    def toolchain_prepare(self, fragment, name, **kwargs): # -> BuildPlan:
        ...
    





"""
This type stub file was generated by pyright.
"""

from collections.abc import Iterable, MutableSequence
from typing import NoReturn, Optional, final
from ._ast import *
from ._ir import Elaboratable, Fragment

__all__ = ["MemoryData", "Memory", "ReadPort", "WritePort", "DummyPort"]


@final
class MemoryData:
    @final
    class Init(MutableSequence):
        """Memory initialization data.

        This is a special container used only for initial contents of memories. It is similar
        to :class:`list`, but does not support inserting or deleting elements; its length is always
        the same as the depth of the memory it belongs to.

        If :py:`shape` is a :ref:`custom shape-castable object <lang-shapecustom>`, then:

        * Each element must be convertible to :py:`shape` via :meth:`.ShapeCastable.const`, and
        * Elements that are not explicitly initialized default to :py:`shape.const(None)`.

        Otherwise (when :py:`shape` is a :class:`.Shape`):

        * Each element must be an :class:`int`, and
        * Elements that are not explicitly initialized default to :py:`0`.
        """
        def __init__(self, elems: Iterable[ValueLike], *, shape: ShapeLike, depth: int) -> None:
            ...
        
        @property
        def shape(self) -> ShapeLike:
            ...
        
        def __getitem__(self, index: int) -> ValueLike:
            ...
        
        def __setitem__(self, index: int, value: ValueLike) -> None:
            ...
        
        def __delitem__(self, index: int) -> NoReturn:
            ...
        
        def insert(self, index: int, value: ValueLike) -> NoReturn:
            ...
        
        def __len__(self) -> int:
            ...
        
        def __repr__(self) -> str:
            ...
        
    
    def __init__(self, *, shape: ShapeLike, depth: int, init: Iterable[ValueLike], src_loc_at: int = ...) -> None:
        ...
    
    def freeze(self) -> None:
        ...
    
    @property
    def shape(self) -> ShapeLike:
        ...
    
    @property
    def depth(self) -> int:
        ...
    
    @property
    def init(self) -> MemoryData.Init:
        ...
    
    @init.setter
    def init(self, init: Iterable[ValueLike]) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    def __getitem__(self, index) -> ValueLike:
        ...
    

class Memory:
    """A word addressable storage.
    Parameters
    ----------
    width : int
        Access granularity. Each storage element of this memory is ``width`` bits in size.
    depth : int
        Word count. This memory contains ``depth`` storage elements.
    init : list of int
        Initial values. At power on, each storage element in this memory is initialized to
        the corresponding element of ``init``, if any, or to zero otherwise.
        Uninitialized memories are not currently supported.
    name : str
        Name hint for this memory. If ``None`` (default) the name is inferred from the variable
        name this ``Signal`` is assigned to.
    attrs : dict
        Dictionary of synthesis attributes.
    Attributes
    ----------
    width : int
    depth : int
    init : list of int
    attrs : dict
    """
    width: int
    depth: int
    attrs: dict

    def __init__(self, *, width: int, depth: int, init: Optional[list[int]] = ..., name: Optional[str] = ..., attrs: dict = ..., simulate: bool = ...) -> None:
        ...
    
    @property
    def init(self) -> list[int]:
        ...
    
    @init.setter
    def init(self, new_init: list[int]) -> None:
        ...
    
    def read_port(self, *, src_loc_at=..., **kwargs) -> ReadPort:
        """Get a read port.

        See :c"""
        ...
    
    def write_port(self, *, src_loc_at=..., **kwargs) -> WritePort:
        """Get a write port.

        See :"""
        ...
    
    def __getitem__(self, index: int) -> ArrayProxy:
        """Simulation only."""
        ...
    


class ReadPort(Elaboratable):
    """A memory read port.

    Paramet"""
    memory: Memory
    domain: str
    transparent: bool
    addr: Signal
    data: Signal
    en: Signal | Const

    def __init__(self, memory, *, domain=..., transparent=..., src_loc_at=...) -> None:
        ...
    
    def elaborate(self, platform): # -> Instance:
        ...
    


class WritePort(Elaboratable):
    """A memory write port.

    Parame"""
    memory: Memory
    domain: str
    granularity: int
    addr: Signal
    data: Signal
    en: Signal

    def __init__(self, memory, *, domain=..., granularity=..., src_loc_at=...) -> None:
        ...
    
    def elaborate(self, platform): # -> Instance:
        ...
    


class DummyPort:
    """Dummy memory port.

    This por"""
    def __init__(self, *, data_width, addr_width, domain=..., name=..., granularity=...) -> None:
        ...
    

class MemoryInstance(Fragment):
    memory: Memory
    read_ports: list[ReadPort]
    write_ports: list[WritePort]
    attrs: dict
    def __init__(self, memory: Memory, read_ports: list[ReadPort], write_ports: list[WritePort]) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

__all__ = ["YosysError", "YosysBinary", "find_yosys"]
from typing import Optional
from pathlib import Path


class YosysError(Exception):
    ...


class YosysWarning(Warning):
    ...


class YosysBinary:
    @classmethod
    def available(cls) -> bool:
        """Check for Yosys availability.

        Returns
        -------
        available : bool
            ``True`` if Yosys is installed, ``False`` otherwise. Installed binary may still not
            be runnable, or might be too old to be useful.
        """
        ...
    
    @classmethod
    def version(cls) -> Optional[tuple[int, int, int]]:
        """Get Yosys version.

        Returns
        -------
        ``None`` if version number could not be determined, or a 3-tuple ``(major, minor, distance)`` if it could.

        major : int
            Major version.
        minor : int
            Minor version.
        distance : int
            Distance to last tag per ``git describe``. May not be exact for system Yosys.
        """
        ...
    
    @classmethod
    def data_dir(cls) -> Path:
        """Get Yosys data directory.

        Returns
        -------
        data_dir : pathlib.Path
            Yosys data directory (also known as "datdir").
        """
        ...
    
    @classmethod
    def run(cls, args: list[str], stdin: str=...) -> str:
        """Run Yosys process.

        Parameters
        ----------
        args : list of str
            Arguments, not including the program name.
        stdin : str
            Standard input.

        Returns
        -------
        stdout : str
            Standard output.

        Exceptions
        ----------
        YosysError
            Raised if Yosys returns a non-zero code. The exception message is the standard error
            output.
        """
        ...
    


class _BuiltinYosys(YosysBinary):
    YOSYS_PACKAGE = ...
    @classmethod
    def available(cls): # -> bool:
        ...
    
    @classmethod
    def version(cls): # -> tuple[int, int, int]:
        ...
    
    @classmethod
    def data_dir(cls): # -> Traversable:
        ...
    
    @classmethod
    def run(cls, args, stdin=..., *, ignore_warnings=..., src_loc_at=...):
        ...
    


class _SystemYosys(YosysBinary):
    YOSYS_BINARY = ...
    @classmethod
    def available(cls) -> bool:
        ...
    
    @classmethod
    def version(cls): # -> tuple[int, int, int] | None:
        ...
    
    @classmethod
    def data_dir(cls) -> Path:
        ...
    
    @classmethod
    def run(cls, args, stdin=..., *, ignore_warnings=..., src_loc_at=...):
        ...
    


def find_yosys(requirement):
    """Find an available Yosys executable of required version.

    Parameters
    ----------
    requirement : function
        Version check. Should return ``True`` if the version is acceptable, ``False`` otherwise.

    Returns
    -------
    yosys_binary : subclass of YosysBinary
        Proxy for running the requested version of Yosys.

    Exceptions
    ----------
    YosysError
        Raised if required Yosys version is not found.
    """
    ...




from .. import *
from amaranth_types import HasElaborate

class RoundRobin(Elaboratable):
    count: int
    requests: Signal
    grant: Signal
    valid: Signal
    def __init__(self, *, count: int):
        ...

    def elaborate(self, platform) -> HasElaborate:
        ...



from ._ast import Shape, unsigned, signed, ShapeCastable, ShapeLike
from ._ast import Value, ValueCastable, ValueLike
from ._ast import Const, C, Mux, Cat, Array, Signal, ClockSignal, ResetSignal, IOValue, IOPort
from ._dsl import SyntaxError, SyntaxWarning, Module
from ._cd import DomainError, ClockDomain
from ._ir import AlreadyElaborated, UnusedElaboratable, Elaboratable, DriverConflict, Fragment, Instance, IOBufferInstance
from ._mem import MemoryData, MemoryInstance, Memory, ReadPort, WritePort, DummyPort
from .rec import Record
from ._xfrm import DomainRenamer, ResetInserter, EnableInserter


__all__ = [
    # _ast
    "Shape", "unsigned", "signed", "ShapeCastable", "ShapeLike",
    "Value", "ValueCastable", "ValueLike",
    "Const", "C", "Mux", "Cat", "Array", "Signal", "ClockSignal", "ResetSignal",
    "IOValue", "IOPort",
    # _dsl
    "SyntaxError", "SyntaxWarning", "Module",
    # _cd
    "DomainError", "ClockDomain",
    # _ir
    "AlreadyElaborated", "UnusedElaboratable", "Elaboratable", "DriverConflict", "Fragment", "Instance", "IOBufferInstance",
    # _mem
    "MemoryData", "MemoryInstance", "Memory", "ReadPort", "WritePort", "DummyPort",
    # _rec
    "Record",
    # _xfrm
    "DomainRenamer", "ResetInserter", "EnableInserter",
]
